# This file is automatically generated by pyo3_stub_gen
# ruff: noqa: E501, F401

import builtins
import typing

class DataManager:
    def get_obj(self, cls_id:builtins.str, partition_id:builtins.int, obj_id:builtins.int) -> typing.Any: ...
    def set_obj(self, obj:ObjectData) -> typing.Any: ...

class InvocationRequest:
    ...

class InvocationResponse:
    def new(self, payload:typing.Sequence[builtins.int], status:builtins.int, header:typing.Mapping[builtins.str, builtins.str]) -> InvocationResponse: ...

class OaasEngine:
    data_manager: DataManager
    def __new__(cls) -> OaasEngine: ...
    def start_server(self, callback:typing.Any) -> None: ...

class ObjectData:
    ...

class ObjectInvocationRequest:
    ...

class ObjectMetadata:
    ...

